@@ -19,67 +19,70 @@ class ParserX86ATT(BaseParser):
 
     def __init__(self):
         super().__init__()
-        self.isa = 'x86'
+        self.isa = "x86"
 
     def construct_parser(self):
         """Create parser for ARM AArch64 ISA."""
-        decimal_number = pp.Combine(
-            pp.Optional(pp.Literal('-')) + pp.Word(pp.nums)
-        ).setResultsName('value')
-        hex_number = pp.Combine(pp.Literal('0x') + pp.Word(pp.hexnums)).setResultsName('value')
+        decimal_number = pp.Combine(pp.Optional(pp.Literal("-")) + pp.Word(pp.nums)).setResultsName(
+            "value"
+        )
+        hex_number = pp.Combine(
+            pp.Optional(pp.Literal("-")) + pp.Literal("0x") + pp.Word(pp.hexnums)
+        ).setResultsName("value")
         # Comment - either '#' or '//' (icc)
-        self.comment = (pp.Literal('#') | pp.Literal('//')) + pp.Group(
+        self.comment = (pp.Literal("#") | pp.Literal("//")) + pp.Group(
             pp.ZeroOrMore(pp.Word(pp.printables))
         ).setResultsName(self.COMMENT_ID)
         # Define x86 assembly identifier
-        relocation = pp.Combine(pp.Literal('@') + pp.Word(pp.alphas))
-        id_offset = pp.Word(pp.nums) + pp.Suppress(pp.Literal('+'))
-        first = pp.Word(pp.alphas + '_.', exact=1)
-        rest = pp.Word(pp.alphanums + '$_.+-')
+        relocation = pp.Combine(pp.Literal("@") + pp.Word(pp.alphas))
+        id_offset = pp.Word(pp.nums) + pp.Suppress(pp.Literal("+"))
+        first = pp.Word(pp.alphas + "_.", exact=1)
+        rest = pp.Word(pp.alphanums + "$_.+-")
         identifier = pp.Group(
-            pp.Optional(id_offset).setResultsName('offset')
-            + pp.Combine(pp.delimitedList(
-                pp.Combine(first + pp.Optional(rest)), delim='::'), joinString='::'
-            ).setResultsName('name')
-            + pp.Optional(relocation).setResultsName('relocation')
-        ).setResultsName('identifier')
+            pp.Optional(id_offset).setResultsName("offset")
+            + pp.Combine(
+                pp.delimitedList(pp.Combine(first + pp.Optional(rest)), delim="::"), joinString="::"
+            ).setResultsName("name")
+            + pp.Optional(relocation).setResultsName("relocation")
+        ).setResultsName("identifier")
         # Label
-        label_rest = pp.Word(pp.alphanums + '$_.+-()')
+        label_rest = pp.Word(pp.alphanums + "$_.+-()")
         label_identifier = pp.Group(
-            pp.Optional(id_offset).setResultsName('offset')
-            + pp.Combine(pp.delimitedList(
-                pp.Combine(first + pp.Optional(label_rest)), delim='::'), joinString='::'
-            ).setResultsName('name')
-            + pp.Optional(relocation).setResultsName('relocation')
-        ).setResultsName('identifier')
+            pp.Optional(id_offset).setResultsName("offset")
+            + pp.Combine(
+                pp.delimitedList(pp.Combine(first + pp.Optional(label_rest)), delim="::"),
+                joinString="::",
+            ).setResultsName("name")
+            + pp.Optional(relocation).setResultsName("relocation")
+        ).setResultsName("identifier")
         numeric_identifier = pp.Group(
-            pp.Word(pp.nums).setResultsName('name')
-            + pp.Optional(pp.oneOf('b f', caseless=True).setResultsName('suffix'))
-        ).setResultsName('identifier')
+            pp.Word(pp.nums).setResultsName("name")
+            + pp.Optional(pp.oneOf("b f", caseless=True).setResultsName("suffix"))
+        ).setResultsName("identifier")
         self.label = pp.Group(
-            (label_identifier | numeric_identifier).setResultsName('name')
-            + pp.Literal(':')
+            (label_identifier | numeric_identifier).setResultsName("name")
+            + pp.Literal(":")
             + pp.Optional(self.comment)
         ).setResultsName(self.LABEL_ID)
         # Register: pp.Regex('^%[0-9a-zA-Z]+{}{z},?')
         self.register = pp.Group(
-            pp.Literal('%')
-            + pp.Word(pp.alphanums).setResultsName('name')
-            + pp.Optional(pp.Literal('(') + pp.Word(pp.nums) + pp.Literal(')'))
+            pp.Literal("%")
+            + pp.Word(pp.alphanums).setResultsName("name")
+            + pp.Optional(pp.Literal("(") + pp.Word(pp.nums) + pp.Literal(")"))
             + pp.Optional(
-                pp.Literal('{')
-                + pp.Optional(pp.Suppress(pp.Literal('%')))
-                + pp.Word(pp.alphanums).setResultsName('mask')
-                + pp.Literal('}')
+                pp.Literal("{")
+                + pp.Optional(pp.Suppress(pp.Literal("%")))
+                + pp.Word(pp.alphanums).setResultsName("mask")
+                + pp.Literal("}")
                 + pp.Optional(
-                    pp.Suppress(pp.Literal('{'))
-                    + pp.Literal('z').setResultsName('zeroing')
-                    + pp.Suppress(pp.Literal('}'))
+                    pp.Suppress(pp.Literal("{"))
+                    + pp.Literal("z").setResultsName("zeroing")
+                    + pp.Suppress(pp.Literal("}"))
                 )
             )
         ).setResultsName(self.REGISTER_ID)
         # Immediate: pp.Regex('^\$(-?[0-9]+)|(0x[0-9a-fA-F]+),?')
-        symbol_immediate = '$'
+        symbol_immediate = "$"
         immediate = pp.Group(
             pp.Literal(symbol_immediate) + (hex_number | decimal_number | identifier)
         ).setResultsName(self.IMMEDIATE_ID)
@@ -88,53 +91,53 @@ class ParserX86ATT(BaseParser):
         offset = pp.Group(identifier | hex_number | decimal_number).setResultsName(
             self.IMMEDIATE_ID
         )
-        scale = pp.Word('1248', exact=1)
+        scale = pp.Word("1248", exact=1)
         # Segment register extension
         segment_extension = (
             hex_number
             ^ pp.Word(pp.nums)
             ^ pp.Group(
-                pp.Optional(offset.setResultsName('offset'))
-                + pp.Literal('(')
-                + pp.Optional(self.register.setResultsName('base'))
-                + pp.Optional(pp.Suppress(pp.Literal(',')))
-                + pp.Optional(self.register.setResultsName('index'))
-                + pp.Optional(pp.Suppress(pp.Literal(',')))
-                + pp.Optional(scale.setResultsName('scale'))
-                + pp.Literal(')')
+                pp.Optional(offset.setResultsName("offset"))
+                + pp.Literal("(")
+                + pp.Optional(self.register.setResultsName("base"))
+                + pp.Optional(pp.Suppress(pp.Literal(",")))
+                + pp.Optional(self.register.setResultsName("index"))
+                + pp.Optional(pp.Suppress(pp.Literal(",")))
+                + pp.Optional(scale.setResultsName("scale"))
+                + pp.Literal(")")
             )
         )
         memory_segmentation = (
-            self.register.setResultsName('base')
-            + pp.Literal(':')
+            pp.Optional(pp.Suppress(pp.Literal("*")))
+            + self.register.setResultsName("base")
+            + pp.Literal(":")
             + segment_extension.setResultsName(self.SEGMENT_EXT_ID)
         )
         # Memory: offset | seg:seg_ext | offset(base, index, scale){mask}
-        memory_abs = (
-            pp.Suppress(pp.Literal('*'))
-            + (offset | self.register).setResultsName('offset')
+        memory_abs = pp.Suppress(pp.Literal("*")) + (offset | self.register).setResultsName(
+            "offset"
         )
         memory = pp.Group(
             (
-                pp.Optional(pp.Suppress(pp.Literal('*')))
-                + pp.Optional(offset.setResultsName('offset'))
-                + pp.Literal('(')
-                + pp.Optional(self.register.setResultsName('base'))
-                + pp.Optional(pp.Suppress(pp.Literal(',')))
-                + pp.Optional(self.register.setResultsName('index'))
-                + pp.Optional(pp.Suppress(pp.Literal(',')))
-                + pp.Optional(scale.setResultsName('scale'))
-                + pp.Literal(')')
+                pp.Optional(pp.Suppress(pp.Literal("*")))
+                + pp.Optional(offset.setResultsName("offset"))
+                + pp.Literal("(")
+                + pp.Optional(self.register.setResultsName("base"))
+                + pp.Optional(pp.Suppress(pp.Literal(",")))
+                + pp.Optional(self.register.setResultsName("index"))
+                + pp.Optional(pp.Suppress(pp.Literal(",")))
+                + pp.Optional(scale.setResultsName("scale"))
+                + pp.Literal(")")
                 + pp.Optional(
-                    pp.Literal('{')
-                    + pp.Optional(pp.Suppress(pp.Literal('%')))
-                    + pp.Word(pp.alphanums).setResultsName('mask')
-                    + pp.Literal('}')
+                    pp.Literal("{")
+                    + pp.Optional(pp.Suppress(pp.Literal("%")))
+                    + pp.Word(pp.alphanums).setResultsName("mask")
+                    + pp.Literal("}")
                 )
             )
             | memory_abs
             | memory_segmentation
-            | (hex_number | pp.Word(pp.nums)).setResultsName('offset')
+            | (hex_number | pp.Word(pp.nums)).setResultsName("offset")
         ).setResultsName(self.MEMORY_ID)
 
         # Directive
@@ -143,23 +146,23 @@ class ParserX86ATT(BaseParser):
         directive_parameter = (
             pp.quotedString
             ^ (
-                pp.Word(pp.printables, excludeChars=',#')
-                + pp.Optional(pp.Suppress(pp.Literal(',')))
+                pp.Word(pp.printables, excludeChars=",#")
+                + pp.Optional(pp.Suppress(pp.Literal(",")))
             )
-            ^ pp.Suppress(pp.Literal(','))
+            ^ pp.Suppress(pp.Literal(","))
         )
         self.directive = pp.Group(
-            pp.Literal('.')
-            + pp.Word(pp.alphanums + '_').setResultsName('name')
-            + pp.ZeroOrMore(directive_parameter).setResultsName('parameters')
+            pp.Literal(".")
+            + pp.Word(pp.alphanums + "_").setResultsName("name")
+            + pp.ZeroOrMore(directive_parameter).setResultsName("parameters")
             + pp.Optional(self.comment)
         ).setResultsName(self.DIRECTIVE_ID)
 
         # Instructions
         # Mnemonic
-        mnemonic = pp.ZeroOrMore(pp.Literal('data16') | pp.Literal('data32')) + pp.Word(
-            pp.alphanums
-        ).setResultsName('mnemonic')
+        mnemonic = pp.ZeroOrMore(pp.Literal("data16") | pp.Literal("data32")) + pp.Word(
+            pp.alphanums + ","
+        ).setResultsName("mnemonic")
         # Combine to instruction form
         operand_first = pp.Group(
             self.register ^ immediate ^ memory ^ identifier ^ numeric_identifier
@@ -167,13 +170,13 @@ class ParserX86ATT(BaseParser):
         operand_rest = pp.Group(self.register ^ immediate ^ memory)
         self.instruction_parser = (
             mnemonic
-            + pp.Optional(operand_first.setResultsName('operand1'))
-            + pp.Optional(pp.Suppress(pp.Literal(',')))
-            + pp.Optional(operand_rest.setResultsName('operand2'))
-            + pp.Optional(pp.Suppress(pp.Literal(',')))
-            + pp.Optional(operand_rest.setResultsName('operand3'))
-            + pp.Optional(pp.Suppress(pp.Literal(',')))
-            + pp.Optional(operand_rest.setResultsName('operand4'))
+            + pp.Optional(operand_first.setResultsName("operand1"))
+            + pp.Optional(pp.Suppress(pp.Literal(",")))
+            + pp.Optional(operand_rest.setResultsName("operand2"))
+            + pp.Optional(pp.Suppress(pp.Literal(",")))
+            + pp.Optional(operand_rest.setResultsName("operand3"))
+            + pp.Optional(pp.Suppress(pp.Literal(",")))
+            + pp.Optional(operand_rest.setResultsName("operand4"))
             + pp.Optional(self.comment)
         )
 
@@ -202,8 +205,8 @@ class ParserX86ATT(BaseParser):
                 self.DIRECTIVE_ID: None,
                 self.COMMENT_ID: None,
                 self.LABEL_ID: None,
-                'line': line,
-                'line_number': line_number,
+                "line": line,
+                "line_number": line_number,
             }
         )
         result = None
@@ -212,7 +215,7 @@ class ParserX86ATT(BaseParser):
         try:
             result = self.process_operand(self.comment.parseString(line, parseAll=True).asDict())
             result = AttrDict.convert_dict(result)
-            instruction_form[self.COMMENT_ID] = ' '.join(result[self.COMMENT_ID])
+            instruction_form[self.COMMENT_ID] = " ".join(result[self.COMMENT_ID])
         except pp.ParseException:
             pass
 
@@ -221,9 +224,9 @@ class ParserX86ATT(BaseParser):
             try:
                 result = self.process_operand(self.label.parseString(line, parseAll=True).asDict())
                 result = AttrDict.convert_dict(result)
-                instruction_form[self.LABEL_ID] = result[self.LABEL_ID]['name']
+                instruction_form[self.LABEL_ID] = result[self.LABEL_ID]["name"]
                 if self.COMMENT_ID in result[self.LABEL_ID]:
-                    instruction_form[self.COMMENT_ID] = ' '.join(
+                    instruction_form[self.COMMENT_ID] = " ".join(
                         result[self.LABEL_ID][self.COMMENT_ID]
                     )
             except pp.ParseException:
@@ -238,12 +241,12 @@ class ParserX86ATT(BaseParser):
                 result = AttrDict.convert_dict(result)
                 instruction_form[self.DIRECTIVE_ID] = AttrDict(
                     {
-                        'name': result[self.DIRECTIVE_ID]['name'],
-                        'parameters': result[self.DIRECTIVE_ID]['parameters'],
+                        "name": result[self.DIRECTIVE_ID]["name"],
+                        "parameters": result[self.DIRECTIVE_ID]["parameters"],
                     }
                 )
                 if self.COMMENT_ID in result[self.DIRECTIVE_ID]:
-                    instruction_form[self.COMMENT_ID] = ' '.join(
+                    instruction_form[self.COMMENT_ID] = " ".join(
                         result[self.DIRECTIVE_ID][self.COMMENT_ID]
                     )
             except pp.ParseException:
@@ -255,7 +258,7 @@ class ParserX86ATT(BaseParser):
                 result = self.parse_instruction(line)
             except pp.ParseException:
                 raise ValueError(
-                    'Could not parse instruction on line {}: {!r}'.format(line_number, line)
+                    "Could not parse instruction on line {}: {!r}".format(line_number, line)
                 )
             instruction_form[self.INSTRUCTION_ID] = result[self.INSTRUCTION_ID]
             instruction_form[self.OPERANDS_ID] = result[self.OPERANDS_ID]
@@ -275,22 +278,22 @@ class ParserX86ATT(BaseParser):
         operands = []
         # Add operands to list
         # Check first operand
-        if 'operand1' in result:
-            operands.append(self.process_operand(result['operand1']))
+        if "operand1" in result:
+            operands.append(self.process_operand(result["operand1"]))
         # Check second operand
-        if 'operand2' in result:
-            operands.append(self.process_operand(result['operand2']))
+        if "operand2" in result:
+            operands.append(self.process_operand(result["operand2"]))
         # Check third operand
-        if 'operand3' in result:
-            operands.append(self.process_operand(result['operand3']))
+        if "operand3" in result:
+            operands.append(self.process_operand(result["operand3"]))
         # Check fourth operand
-        if 'operand4' in result:
-            operands.append(self.process_operand(result['operand4']))
+        if "operand4" in result:
+            operands.append(self.process_operand(result["operand4"]))
         return_dict = AttrDict(
             {
-                self.INSTRUCTION_ID: result['mnemonic'],
+                self.INSTRUCTION_ID: result["mnemonic"].split(",")[0],
                 self.OPERANDS_ID: operands,
-                self.COMMENT_ID: ' '.join(result[self.COMMENT_ID])
+                self.COMMENT_ID: " ".join(result[self.COMMENT_ID])
                 if self.COMMENT_ID in result
                 else None,
             }
@@ -311,23 +314,23 @@ class ParserX86ATT(BaseParser):
         return operand
 
     def process_directive(self, directive):
-        directive_new = {'name': directive['name'], 'parameters': []}
-        if 'parameters' in directive:
-            directive_new['parameters'] = directive['parameters']
-        if 'comment' in directive:
-            directive_new['comment'] = directive['comment']
+        directive_new = {"name": directive["name"], "parameters": []}
+        if "parameters" in directive:
+            directive_new["parameters"] = directive["parameters"]
+        if "comment" in directive:
+            directive_new["comment"] = directive["comment"]
         return AttrDict({self.DIRECTIVE_ID: directive_new})
 
     def process_memory_address(self, memory_address):
         """Post-process memory address operand"""
         # Remove unecessarily created dictionary entries during memory address parsing
-        offset = memory_address.get('offset', None)
-        base = memory_address.get('base', None)
-        index = memory_address.get('index', None)
-        scale = 1 if 'scale' not in memory_address else int(memory_address['scale'])
+        offset = memory_address.get("offset", None)
+        base = memory_address.get("base", None)
+        index = memory_address.get("index", None)
+        scale = 1 if "scale" not in memory_address else int(memory_address["scale"])
         if isinstance(offset, str) and base is None and index is None:
-            offset = {'value': offset}
-        new_dict = AttrDict({'offset': offset, 'base': base, 'index': index, 'scale': scale})
+            offset = {"value": offset}
+        new_dict = AttrDict({"offset": offset, "base": base, "index": index, "scale": scale})
         # Add segmentation extension if existing
         if self.SEGMENT_EXT_ID in memory_address:
             new_dict[self.SEGMENT_EXT_ID] = memory_address[self.SEGMENT_EXT_ID]
@@ -336,12 +339,12 @@ class ParserX86ATT(BaseParser):
     def process_label(self, label):
         """Post-process label asm line"""
         # remove duplicated 'name' level due to identifier
-        label['name'] = label['name'][0]['name']
+        label["name"] = label["name"][0]["name"]
         return AttrDict({self.LABEL_ID: label})
 
     def process_immediate(self, immediate):
         """Post-process immediate operand"""
-        if 'identifier' in immediate:
+        if "identifier" in immediate:
             # actually an identifier, change declaration
             return immediate
         # otherwise nothing to do
@@ -350,15 +353,15 @@ class ParserX86ATT(BaseParser):
     def get_full_reg_name(self, register):
         """Return one register name string including all attributes"""
         # nothing to do
-        return register['name']
+        return register["name"]
 
     def normalize_imd(self, imd):
         """Normalize immediate to decimal based representation"""
-        if 'value' in imd:
-            if imd['value'].lower().startswith('0x'):
+        if "value" in imd:
+            if imd["value"].lower().startswith("0x"):
                 # hex, return decimal
-                return int(imd['value'], 16)
-            return int(imd['value'], 10)
+                return int(imd["value"], 16)
+            return int(imd["value"], 10)
         # identifier
         return imd
 
@@ -373,8 +376,8 @@ class ParserX86ATT(BaseParser):
     def is_reg_dependend_of(self, reg_a, reg_b):
         """Check if ``reg_a`` is dependent on ``reg_b``"""
         # Normalize name
-        reg_a_name = reg_a['name'].upper()
-        reg_b_name = reg_b['name'].upper()
+        reg_a_name = reg_a["name"].upper()
+        reg_b_name = reg_b["name"].upper()
 
         # Check if they are the same registers
         if reg_a_name == reg_b_name:
@@ -388,13 +391,13 @@ class ParserX86ATT(BaseParser):
             return False
         # Check basic GPRs
         gpr_groups = {
-            'A': ['RAX', 'EAX', 'AX', 'AH', 'AL'],
-            'B': ['RBX', 'EBX', 'BX', 'BH', 'BL'],
-            'C': ['RCX', 'ECX', 'CX', 'CH', 'CL'],
-            'D': ['RDX', 'EDX', 'DX', 'DH', 'DL'],
-            'SP': ['RSP', 'ESP', 'SP', 'SPL'],
-            'SRC': ['RSI', 'ESI', 'SI', 'SIL'],
-            'DST': ['RDI', 'EDI', 'DI', 'DIL']
+            "A": ["RAX", "EAX", "AX", "AH", "AL"],
+            "B": ["RBX", "EBX", "BX", "BH", "BL"],
+            "C": ["RCX", "ECX", "CX", "CH", "CL"],
+            "D": ["RDX", "EDX", "DX", "DH", "DL"],
+            "SP": ["RSP", "ESP", "SP", "SPL"],
+            "SRC": ["RSI", "ESI", "SI", "SIL"],
+            "DST": ["RDI", "EDI", "DI", "DIL"],
         }
         if self.is_basic_gpr(reg_a):
             if self.is_basic_gpr(reg_b):
@@ -405,8 +408,8 @@ class ParserX86ATT(BaseParser):
             return False
 
         # Check other GPRs
-        ma = re.match(r'R([0-9]+)[DWB]?', reg_a_name)
-        mb = re.match(r'R([0-9]+)[DWB]?', reg_b_name)
+        ma = re.match(r"R([0-9]+)[DWB]?", reg_a_name)
+        mb = re.match(r"R([0-9]+)[DWB]?", reg_b_name)
         if ma and mb and ma.group(1) == mb.group(1):
             return True
 
@@ -415,9 +418,8 @@ class ParserX86ATT(BaseParser):
 
     def is_basic_gpr(self, register):
         """Check if register is a basic general purpose register (ebi, rax, ...)"""
-        if (
-            any(char.isdigit() for char in register['name'])
-            or any(register['name'].lower().startswith(x) for x in ['mm', 'xmm', 'ymm', 'zmm'])
+        if any(char.isdigit() for char in register["name"]) or any(
+            register["name"].lower().startswith(x) for x in ["mm", "xmm", "ymm", "zmm"]
         ):
             return False
         return True
@@ -428,13 +430,13 @@ class ParserX86ATT(BaseParser):
             return False
         if self.is_basic_gpr(register):
             return True
-        return re.match(r'R([0-9]+)[DWB]?', register['name'], re.IGNORECASE)
+        return re.match(r"R([0-9]+)[DWB]?", register["name"], re.IGNORECASE)
 
     def is_vector_register(self, register):
         """Check if register is a vector register"""
         if register is None:
             return False
-        if register['name'].rstrip(string.digits).lower() in ['mm', 'xmm', 'ymm', 'zmm']:
+        if register["name"].rstrip(string.digits).lower() in ["mm", "xmm", "ymm", "zmm"]:
             return True
         return False
 
@@ -443,7 +445,7 @@ class ParserX86ATT(BaseParser):
         if register is None:
             return False
         if self.is_gpr(register):
-            return 'gpr'
+            return "gpr"
         elif self.is_vector_register(register):
-            return register['name'].rstrip(string.digits).lower()
+            return register["name"].rstrip(string.digits).lower()
         raise ValueError
